#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Stefan Roellin <stefan()roellin-baumann.ch>'
__copyright__ = 'Copyright 2015 Stefan Roellin'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from future import standard_library
standard_library.install_aliases()

import http.client, urllib.request, urllib.parse, urllib.error


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    data = {'Message': item.message.encode('utf-8'),
            'Origin': item.title,
            'Timeout': item.config.get('timeout', 5),
            'Address': item.addrs[1],
            'Progress': -1
            }
    if item.image != None:
        data['Image'] = item.image

    http_handler = http.client.HTTPConnection(item.addrs[0])

    try:
        http_handler.request("POST", 'Myth/SendNotification',
                             headers={'Content-type': "application/x-www-form-urlencoded", "Accept": "text/plain"},
                             body=urllib.parse.urlencode(data))
    except (SSLError, HTTPException) as e:
        srv.logging.warn("mythtv notification failed: %s" % e)
        return False            

    response = http_handler.getresponse()

    srv.logging.debug("Reponse: %s, %s" % (response.status, response.reason))

    return True 



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Ben Jones <ben.jones12()gmail.com>'
__copyright__ = 'Copyright 2014 Ben Jones'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from future import standard_library
standard_library.install_aliases()

import urllib.request, urllib.parse, urllib.error
import urllib.request, urllib.error, urllib.parse
import base64
try:
    import simplejson as json
except ImportError:
    import json


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    xbmchost = item.addrs[0]
    xbmcusername = None
    xbmcpassword = None

    if len(item.addrs) == 3:
        xbmcusername = item.addrs[1]
        xbmcpassword = item.addrs[2]

    title    = item.title
    message  = item.message
    image    = item.image

    jsonparams = {
        "jsonrpc" : "2.0",
        "method"  : "GUI.ShowNotification",
        "id"      : 1,
        "params"  : {
            "title"       : title,
            "message"     : message,
            "image"       : image,
            "displaytime" : 10000
        }
    }
    jsoncommand = json.dumps(jsonparams)

    url = 'http://%s/jsonrpc' % (xbmchost)
    try:
        srv.logging.debug("Sending XBMC notification to %s [%s]..." % (item.target, xbmchost))
        req = urllib.request.Request(url, jsoncommand)
        req.add_header("Content-type", "application/json")
        if xbmcpassword is not None:
            base64string = base64.encodestring('%s:%s' % (xbmcusername, xbmcpassword))[:-1]
            authheader = "Basic %s" % base64string
            req.add_header("Authorization", authheader)
        response = urllib.request.urlopen(req, timeout = 2)
        srv.logging.debug("Successfully sent XBMC notification")
    except urllib.error.URLError as e:
        srv.logging.error("URLError: %s" % e)
        return False
    except Exception as e:
        srv.logging.error("Error sending XBMC notification to %s [%s]: %s" % (item.target, xbmchost, e))
        return False

    return True



# -*- coding: utf-8 -*-

__author__    = 'Andreas Motl <andreas.motl@panodata.org>'
__copyright__ = 'Copyright 2020 Andreas Motl'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

# https://github.com/caronc/apprise#developers
import apprise


def plugin(srv, item):
    """Send a message to Apprise plugin(s)."""

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    addresses = item.addrs

    if not addresses:
        srv.logging.warn("Skipped sending notification to Apprise %s, "
                         "no addresses configured" % (item.target))
        return False

    sender = item.config.get('sender')
    sender_name = item.config.get('sender_name')
    baseuri = item.config['baseuri']
    title = item.title
    body = item.message

    try:
        srv.logging.debug("Sending notification to Apprise %s, addresses: %s" % (item.target, addresses))
        to = ','.join(addresses)

        # Create an Apprise instance.
        apobj = apprise.Apprise()

        # Add notification services by server url.
        uri = '{baseuri}?from={sender}&to={to}'.format(baseuri=baseuri, sender=sender, to=to)
        if sender_name:
            uri += '&name={sender_name}'.format(sender_name=sender_name)
        apobj.add(uri)

        # Submit notification.
        outcome = apobj.notify(
            body=body,
            title=title,
        )

        if outcome:
            srv.logging.info("Successfully sent message using Apprise")
            return True

        else:
            srv.logging.error("Sending message using Apprise failed")
            return False

    except Exception as e:
        srv.logging.error("Error sending message to %s: %s" % (item.target, e))
        return False



# -*- coding: utf-8 -*-
# (c) 2014-2018 The mqttwarn developers

__version__   = '0.14.0'

__author__    = 'Jan-Piet Mens <jpmens()gmail.com>, Ben Jones <ben.jones12()gmail.com>'
__copyright__ = 'Copyright 2014-2018 Jan-Piet Mens'
__license__   = 'Eclipse Public License - v 2.0 (http://www.eclipse.org/legal/epl-2.0/)'



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Ben Jones <ben.jones12()gmail.com>'
__copyright__ = 'Copyright 2014 Ben Jones'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

from future import standard_library
standard_library.install_aliases()

from xmlrpc.client import ServerProxy
import urllib.request, urllib.parse, urllib.error


def plugin(srv, item):

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    host      = item.config['host']
    port      = item.config['port']
    username  = item.config['username']
    password  = item.config['password']
    ttsurl    = item.config['ttsurl']
    ttsparams = item.config['ttsparams']
 
    gateway   = item.addrs[0]
    number    = item.addrs[1]    
    title     = item.title

    if ttsurl.startswith('http://'):
        ttsurl = ttsurl[7:]
    elif ttsurl.startswith('https://'):
        ttsurl = ttsurl[8:]

    if ttsparams is not None:
        for key in list(ttsparams.keys()):

            # { 'q' : '@message' }
            # Quoted field, starts with '@'. Do not use .format, instead grab
            # the item's [message] and inject as parameter value.
            if ttsparams[key].startswith('@'):         # "@message"
                ttsparams[key] = item.get(ttsparams[key][1:], "NOP")

            else:
                try:
                    ttsparams[key] = ttsparams[key].format(**item.data).encode('utf-8')
                except Exception as e:
                    srv.logging.debug("Parameter %s cannot be formatted: %s" % (key, e))
                    return False

    try:
        # TTS service
        shout_url = "shout://%s" % ttsurl
        if ttsparams is not None:
            if not shout_url.endswith('?'):
                shout_url = shout_url + '?'
            shout_url = shout_url + urllib.parse.urlencode(ttsparams)
        # debugging
        srv.logging.debug("Shout URL: %s" % shout_url)
        # Freeswitch API
        server = ServerProxy("http://%s:%s@%s:%d" % (username, password, host, port))
        # channel variables we need to setup the call
        channel_vars = "{ignore_early_media=true,originate_timeout=60,origination_caller_id_name='" + title + "'}"
        # originate the call
        server.freeswitch.api("originate", channel_vars + gateway + number + " &playback(" + shout_url + ")")
    except Exception as e:
        srv.logging.error("Error originating Freeswitch VOIP call to %s via %s%s: %s" % (item.target, gateway, number, e))
        return False

    return True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__author__    = 'Fabian Affolter <fabian()affolter-engineering.ch>'
__copyright__ = 'Copyright 2014 Fabian Affolter'
__license__   = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import xmpp


def plugin(srv, item):
    """Send a message to XMPP recipient(s)."""

    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)

    xmpp_addresses = item.addrs
    sender = item.config['sender']
    password = item.config['password']
    text = item.message

    if not xmpp_addresses:
        srv.logging.warn("Skipped sending XMPP notification to %s, "
                         "no addresses configured" % (item.target))
        return False

    try:
        srv.logging.debug("Sending XMPP notification to %s, addresses: %s" % (item.target, xmpp_addresses))
        for target in xmpp_addresses:
            jid = xmpp.protocol.JID(sender)
            connection = xmpp.Client(jid.getDomain(),debug=[])
            connection.connect()
            connection.auth(jid.getNode(), password, resource=jid.getResource())
            connection.send(xmpp.protocol.Message(target, text))
        srv.logging.debug("Successfully sent message")
    except Exception as e:
        srv.logging.error("Error sending message to %s: %s" % (item.target, e))
        return False

    return True


def xmpppy_monkeypatch_ssl():
    """
    Mitigate "AttributeError: '_ssl._SSLSocket' object has no attribute 'issuer'"

    Monkey-patched _startSSL method from
    https://raw.githubusercontent.com/freebsd/freebsd-ports/master/net-im/py-xmpppy/files/patch-xmpp-transports.py
    """
    import ssl

    def _startSSL(self):
        """ Immidiatedly switch socket to TLS mode. Used internally."""
        """ Here we should switch pending_data to hint mode."""
        tcpsock=self._owner.Connection
        tcpsock._sslObj    = ssl.wrap_socket(tcpsock._sock, None, None)
        tcpsock._sslIssuer = tcpsock._sslObj.getpeercert().get('issuer')
        tcpsock._sslServer = tcpsock._sslObj.getpeercert().get('server')
        tcpsock._recv = tcpsock._sslObj.read
        tcpsock._send = tcpsock._sslObj.write

        tcpsock._seen_data=1
        self._tcpsock=tcpsock
        tcpsock.pending_data=self.pending_data
        tcpsock._sock.setblocking(0)

        self.starttls='success'

    from xmpp.transports import TLS
    TLS._startSSL = _startSSL



#!/usr/bin/env python2
# -*- coding: utf-8 -*-
from six import string_types

__author__ = 'David Ventura'
__copyright__ = 'Copyright 2016 David Ventura'
__license__ = 'Eclipse Public License - v 1.0 (http://www.eclipse.org/legal/epl-v10.html)'

import os
import json
import paramiko
from pipes import quote


def credentials(host, user=None, pwd=None, port=22):
    c = {}
    if user is None and pwd is None:
        config = paramiko.SSHConfig()
        p = os.path.expanduser('~') + '/.ssh/config'
        config.parse(open(p))
        o = config.lookup(host)

        ident = o['identityfile']
        if type(ident) is list:
            ident = ident[0]
        if 'port' not in o:
            o['port'] = port
        c = {'hostname': o['hostname'], 'port': int(o['port']), 'username': o['user'], 'key_filename': ident}
    else:
        c = {'hostname': host, 'port': port, 'username': user, 'password': pwd}

    return c


def plugin(srv, item):
    srv.logging.debug("*** MODULE=%s: service=%s, target=%s", __file__, item.service, item.target)
    host = item.config["host"]
    port = 22
    user = None
    pwd = None

    if 'port' in item.config:
        port = item.config["port"]
    if 'user' in item.config:
        user = item.config["user"]
    if 'pass' in item.config:
        pwd = item.config["pass"]

    command = item.addrs[0]

    args = json.loads(item.payload)["args"]
    if type(args) is list and len(args) == 1:
        args = args[0]

    if isinstance(args, list):
        args = tuple([quote(v) for v in args])  # escape the shell args
    elif isinstance(args, string_types):
        args = (quote(args),)

    command = command % args

    ssh = paramiko.SSHClient()
    ssh.load_system_host_keys()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())

    try:
        c = credentials(host, user=user, pwd=pwd, port=port)
        ssh.connect(**c)
        _, stdout, stderr = ssh.exec_command(command)

    except Exception as e:
        srv.logging.warning("Cannot run command %s on host %s" % (command, host))
        srv.logging.warning("%s" % e)
        return False

    return True
